/*
 * drvMcaXmap.c
 *
 *  Created on: 15-Sep-2008
 *      Author: up45
 */
#include "drvXmap.h"

/* start or stop acquisition on the given channel (or all if required) */
static asynStatus xmap_mca_startacq(xmap_driver_t* pPvt, asynUser* pasynUser, epicsUInt32 startAcq);
/* get the run-time status from the xmap (live time, real time, acquisition state and so on) */
static asynStatus xmap_mca_status(xmap_driver_t* pPvt, asynUser* pasynUser);
/* set the number of bins in the MCA spectrum. */
static asynStatus xmap_mca_nbins(xmap_driver_t* pPvt, asynUser* pasynUser, epicsInt32 nbins);
/* request the xMAP for one modules worth of MCA data (4 channels in sync)*/
static asynStatus xmap_mca_collect_module_data(xmap_driver_t* pPvt, asynUser* pasynUser);
/* Change the preset run until type and value */
static asynStatus xmap_mca_change_preset(xmap_driver_t* pPvt, asynUser* pasynUser, epicsUInt32 type, epicsFloat64 presetvalue);

asynStatus xmap_mca_int_write (xmap_driver_t* pPvt, asynUser *pasynUser, epicsInt32 value)
{
	asynStatus status = asynSuccess;
	epicsInt32 channel;
	epicsInt32 reason = pasynUser->reason;
	pasynManager->getAddr(pasynUser, &channel);

	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_int_write MCA reason: %d value: %d\n", reason, value);

	switch (reason)
	{
	case xmap_mca_collect:
		status = xmap_mca_collect_module_data(pPvt,pasynUser);
		break;
	case xmap_mca_stat:
		status = xmap_mca_status(pPvt, pasynUser);
		break;
	case xmap_mca_preset_type:
		status = xmap_mca_change_preset( pPvt, pasynUser, value, pPvt->preset_value);
		break;
	case xmap_start_all:
		pPvt->start_mode = XMAP_START_ALL;
		status = xmap_mca_startacq(pPvt, pasynUser, value);
		break;
	case mcaStartAcquire:
		value = TRUE;
	case mcaStopAcquire:
		pPvt->start_mode = XMAP_START_MODULE;
		status = xmap_mca_startacq(pPvt, pasynUser, value);
		break;
	case mcaErase:
		pPvt->resume = FALSE;
		break;
	case mcaReadStatus:
		/* This is called by the MCA record when writing to the PROC
		 * field or when the record processes in general.
		 * However, since we read status for the entire module in xmap_mca_stat
		 * we don't perform any action here. */
		break;
	case mcaChannelAdvanceInternal:
		break;
	case mcaChannelAdvanceExternal:
		break;
	case mcaNumChannels:
		status = xmap_mca_nbins(pPvt, pasynUser, value);
		break;
	case mcaModePHA:
		break;
	case mcaModeMCS:
		break;
	case mcaModeList:
		break;
	case mcaSequence:
		break;
	case mcaPrescale:
		break;
	case mcaPresetSweeps:
		break;
	case mcaPresetLowChannel:
		break;
	case mcaPresetHighChannel:
		break;
	default:
		asynPrint(pasynUser, ASYN_TRACE_ERROR,
				"drvMcaXmap::xmap_mca_int_write ERROR: reason %d is not implemented for mca int write.\n",
				reason);
		status = asynError;
		break;
	}
	return status;
}

asynStatus xmap_mca_int_read (xmap_driver_t* pPvt, asynUser *pasynUser, epicsInt32 *value)
{
	asynStatus status = asynSuccess;
	epicsInt32 reason = pasynUser->reason;
	epicsInt32 addr, modFirstCh, xch;
	xmap_channel_t *chan;
	pasynManager->getAddr(pasynUser, &addr);
	chan = &(pPvt->chan[addr]);
	xch = pPvt->xch[addr];
	modFirstCh = pPvt->xch[0];

	switch(reason)
	{
		case mcaAcquiring:
			asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_int_read reason mcaAcquiring (old) value: %d\n", *value);
			*value = chan->acquiring;
			break;
		case xmap_mca_triggers:
			*value = (epicsInt32)chan->triggers;
			break;
		case xmap_mca_preset_type:
			*value = pPvt->preset_type;
			break;
		case xmap_mca_acqstat:
			*value = (epicsInt32)(chan->run_active_status);
			break;
		default:
			asynPrint(pasynUser, ASYN_TRACE_ERROR,
					"drvMcaXmap::xmap_mca_int_read ERROR: reason %d is not implemented for mca int read.\n",
					reason);
			status = asynError;
			break;
	}
	return status;
}

asynStatus xmap_mca_int_array_read (xmap_driver_t* pPvt, asynUser *pasynUser, epicsInt32 *data, size_t maxChans, size_t *nactual)
{
	asynStatus status = asynSuccess;
	epicsInt32 reason = pasynUser->reason;
	epicsInt32 addr, modFirstCh, xch;
	xmap_channel_t *chan;
	pasynManager->getAddr(pasynUser, &addr);
	chan = &(pPvt->chan[addr]);
	xch = pPvt->xch[addr];
	modFirstCh = pPvt->xch[0];

/*	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_int_array_read globalCh=%d maxChans=%d\n",
			xch, maxChans);
*/
	if (reason != mcaData)
	{
		asynPrint(pasynUser, ASYN_TRACE_ERROR,
				"drvMcaXmap::xmap_mca_int_array_read ERROR: reason %d is not implemented for mca int array read.\n",
				reason);
		return asynError;
	}
	*nactual = 0;
	/* Read one MCA spectrum out from the latched channels spectrum here */
	memcpy(data, chan->mcaChData, chan->nBins * sizeof(epicsInt32));
	*nactual = chan->nBins;
	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_int_array_read NELM: %d, nbins: %d, dataptr: 0x%p *nactual: %d\n",
	       maxChans, chan->nBins, chan->mcaChData, *nactual);
	return status;
}

asynStatus xmap_int_array_write (xmap_driver_t* pPvt, asynUser *pasynUser, epicsInt32 *data, size_t maxChans)
{
	asynStatus status = asynSuccess;
	epicsInt32 reason = pasynUser->reason;
	if (reason != mcaData)
	{
		asynPrint(pasynUser, ASYN_TRACE_ERROR,
				"drvMcaXmap::xmap_mca_int_array_read ERROR: reason %d is not implemented for mca int array write.\n",
				reason);
		return asynError;
	}
	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_int_array_write - NELM: %d - this command is not currently implemented\n",
			maxChans);
	return status;
}

asynStatus xmap_mca_float_write (xmap_driver_t* pPvt, asynUser *pasynUser, epicsFloat64 value)
{
	asynStatus status = asynSuccess;
	epicsInt32 reason = pasynUser->reason;
	epicsInt32 channel;
	pasynManager->getAddr(pasynUser, &channel);
	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_float_write ch: %d reason: %d value: %.3f\n",
			channel, reason, value);

	switch (reason)
	{
	case mcaDwellTime:
		break;
	case mcaPresetLiveTime:
		status = xmap_mca_change_preset(pPvt, pasynUser, XMAP_PRESET_LIVE, value);
		break;
	case mcaPresetRealTime:
		status = xmap_mca_change_preset(pPvt, pasynUser, XMAP_PRESET_REAL, value);
		break;
	case mcaPresetCounts:
		status = xmap_mca_change_preset(pPvt, pasynUser, XMAP_PRESET_EVENTS, value);
		break;
	case xmap_mca_preset_value:
		status = xmap_mca_change_preset(pPvt, pasynUser, pPvt->preset_type, value);
		break;
	default:
		asynPrint(pasynUser, ASYN_TRACE_ERROR,
				"drvMcaXmap::xmap_mca_float_write ERROR: reason %d is not implemented for mca float write.\n",
				reason);
		status = asynError;
		break;
	}
	return status;
}

asynStatus xmap_mca_float_read (xmap_driver_t* pPvt, asynUser *pasynUser, epicsFloat64 *value)
{
	asynStatus status = asynSuccess;
	epicsInt32 reason = pasynUser->reason;
	epicsInt32 channel;
	epicsFloat64 oldval;
	pasynManager->getAddr(pasynUser, &channel);
	oldval = *value;

	switch (reason)
	{
	case mcaDwellTime:
		break;
	case mcaElapsedLiveTime:
		*value = pPvt->chan[channel].trigger_livetime;
		break;
	case mcaElapsedRealTime:
		*value = pPvt->chan[channel].realtime;
		break;
	case mcaElapsedCounts:
		*value = pPvt->chan[channel].events;
		break;
	case xmap_mca_preset_value:
		*value = pPvt->preset_value;
		break;
	default:
		asynPrint(pasynUser, ASYN_TRACE_ERROR,
				"drvMcaXmap::xmap_mca_float_read ERROR: reason %d is not implemented for mca float read.\n",
				reason);
		status = asynError;
		break;
	}
	asynPrint(pasynUser, ASYN_TRACEIO_DEVICE, "drvMcaXmap::xmap_mca_float_read ch: %d reason: %d value: %.3f (was %.3f)\n",
			channel, reason, *value, oldval);

	return status;
}






/* start or stop acquisition on the given channel (or all if required) */
static asynStatus xmap_mca_startacq(xmap_driver_t* pPvt, asynUser* pasynUser, epicsUInt32 startAcq)
{
	asynStatus status = asynSuccess;
	epicsInt32 addr, currentstate, xiastatus, tmpAcq = FALSE, i, start_ch;
	xmap_channel_t *chan;
	pasynManager->getAddr(pasynUser, &addr);
	chan = &(pPvt->chan[addr]);
	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_startacq ch=%d startAcq=%d start_mode=%d\n", addr, startAcq, pPvt->start_mode);

	/* check if this call is to start/stop all channels or just the one */
	if (pPvt->start_mode == XMAP_START_ALL)
	{
		start_ch = XMAP_ALL;
		currentstate = pPvt->acquiring;
	} else
	{
		start_ch = pPvt->xch[addr];
		currentstate = chan->acquiring;
	}

	/* check if acquisition is already running... */
	if (currentstate == TRUE && startAcq == TRUE)
	{
		asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_startacq Acquisition on channel %d already running\n", addr);
		return status;
	}
	if (currentstate == FALSE && startAcq == FALSE)
	{
		asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_startacq Acquisition on channel %d already stopped\n", addr);
		return status;
	}

	if (startAcq == TRUE)
	{
    	asynPrint(pasynUser, ASYN_TRACE_FLOW, "    Starting run! start_ch=%d resume=%d\n", start_ch, pPvt->resume);
		xiastatus = xiaStartRun(start_ch, pPvt->resume);		/* Start the run!  */
		status = xmap_xia_checkError( pasynUser, xiastatus, "xiaStartRun()" );
		if (status == asynError) return status;
	} else
	{
    	asynPrint(pasynUser, ASYN_TRACE_FLOW, "    Stopping run! ch=%d\n", start_ch);
		xiastatus = xiaStopRun(start_ch);					/* Stop the run */
		status = xmap_xia_checkError( pasynUser, xiastatus, "xiaStartRun()" );
		if (status == asynError) return status;
	}

	/* Set the appropriate flags */
	if (start_ch == XMAP_ALL)
		/* ... for all the channels if all are now acquiring */
		for (i = 0; i < XMAP_NCHANS_MODULE; i++) pPvt->chan[i].acquiring = startAcq;
	else /* ... or just for the individual module channels we are using */
		chan->acquiring = startAcq; /* TODO: need to set all channels on a module acquiring...*/

	/* If we started acquisition on any channel we set the global acq flag */
	if ( startAcq == TRUE ) pPvt->acquiring = startAcq;
	else
	{
		/* ... otherwise (if we stopped acquisition) we run through all
		 * channels to see if any of them are still acquiring */
		for (i = 0, tmpAcq = FALSE; i < XMAP_NCHANS_MODULE; i++)
		{
			tmpAcq = pPvt->chan[i].acquiring;
			if (tmpAcq == TRUE) break;
		}
	}
	/* ... so if all of them have stopped acquiring we clear the
	 * global acquisition flag */
	pPvt->acquiring = tmpAcq;
	if (pPvt->acquiring == FALSE) pPvt->resume = TRUE;
	return status;
}

/* get the run-time status from the xmap
 * live time, real time , acquisition state and so on */
static asynStatus xmap_mca_status(xmap_driver_t* pPvt, asynUser* pasynUser)
{
	asynStatus status = asynSuccess;
	epicsInt32 addr, xch, xiastatus, i, j;
	epicsUInt32 tmpRunStatus, tmpAcqStat = 0;
	xmap_channel_t *chan;
	pasynManager->getAddr(pasynUser, &addr);
	chan = &(pPvt->chan[addr]);
	xch = pPvt->xch[addr];

	/*asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_status ch: %d module: %d\n", xch, pPvt->moduleNo);
*/
	/* Get the statistics for the entire module (4ch) in one go.
	 * Store the values in the respective channels structure.
	 */
	xiastatus = xiaGetRunData(xch, "module_statistics", (void*)pPvt->tmpModuleStats);
	status = xmap_xia_checkError(pasynUser, xiastatus, "module_statistics");
	if (status == asynError) return status;
	asynPrint(pasynUser, ASYN_TRACE_FLOW, "    Module stats: %.3f %.3f %.3f %.3f\n",
			pPvt->tmpModuleStats[0],pPvt->tmpModuleStats[1],pPvt->tmpModuleStats[2],pPvt->tmpModuleStats[3]);

	/* Get the acquisition status (run_active) for all four channels
	 * on the module. Set the respective channels state in the chan structure */
	for(i = 0; i < XMAP_NCHANS_MODULE; i++)
	{
		chan = &(pPvt->chan[i]);
		xch = pPvt->xch[i];
		xiastatus = xiaGetRunData(xch, "run_active", (void*)&tmpAcqStat);
		status = xmap_xia_checkError(pasynUser, xiastatus, "run_active");
		if (status == asynError) return status;
		asynPrint(pasynUser, ASYN_TRACE_FLOW, "    run_active [%d]: %d (last: %d)\n", xch, tmpAcqStat, chan->run_active_status);

		/* If the result is non-zero that means xmap is running or some part of the handel library
		 * thinks it is anyway... */
		switch(tmpAcqStat)
		{
			/* definitely stopped... */
			case 0:
				chan->acquiring = FALSE;
				/* check all the channels to see if any are still running */
				tmpRunStatus = 0;
				for (j = 0; j < XMAP_NCHANS_MODULE; j++)
				{
					tmpRunStatus = pPvt->chan[j].acquiring;
					if (tmpRunStatus != 0) break;
				}
				pPvt->acquiring = tmpRunStatus;
				/* if we have really stopped all acquiring then we set the
				 * resume flag back to TRUE. When user commands mcaErase
				 * it will be set TRUE again.    */
				if (pPvt->acquiring == FALSE) pPvt->resume = TRUE;
				break;
				/* The hardware thinks it's running... */
			case (XIA_RUN_HARDWARE ^ XIA_RUN_HANDEL):
				chan->acquiring = TRUE;  /* Set the specific channels state */
				pPvt->acquiring = TRUE;  /* Set the overall acquiring state which
												means that *some* channel is active */
				break;
			case XIA_RUN_HARDWARE:
			case XIA_RUN_HANDEL:
				/* If we already had this status in the last check, we do something about it */
/*				if (chan->run_active_status == tmpAcqStat)
				{
					asynPrint(pasynUser, ASYN_TRACE_FLOW, "    UPS: status = %d stopping acquisition.\n", chan->run_active_status);
					pPvt->start_mode = XMAP_START_MODULE;
					status = xmap_mca_startacq(pPvt, pasynUser, 0);
				}
*/				break;
			default:
				asynPrint(pasynUser, ASYN_TRACE_ERROR, "WARNING: unhandled status: %d\n", tmpAcqStat);
				break;
		}
		chan->run_active_status = tmpAcqStat;
	}
	asynPrint(pasynUser, ASYN_TRACE_FLOW, "    Runactive: #0: %d #1: %d #2: %d #3: %d\n",
			pPvt->chan[0].acquiring,pPvt->chan[1].acquiring,
			pPvt->chan[2].acquiring,pPvt->chan[3].acquiring);

	/* Store the module statistics in their respective channels */
	for (i = 0; i < XMAP_NCHANS_MODULE; i++)
	{
		chan = &(pPvt->chan[i]);
		chan->realtime          = pPvt->tmpModuleStats[0];
		chan->trigger_livetime  = pPvt->tmpModuleStats[1];
		chan->energy_livetime   = pPvt->tmpModuleStats[2];
		chan->triggers          = pPvt->tmpModuleStats[3];
		chan->events            = pPvt->tmpModuleStats[4];
		chan->input_count_rate  = pPvt->tmpModuleStats[5];
		chan->output_count_rate = pPvt->tmpModuleStats[6];
	}
	return status;
}

/* set the number of bins in the MCA spectrum. */
static asynStatus xmap_mca_nbins(xmap_driver_t* pPvt, asynUser* pasynUser, epicsInt32 nbins)
{
	asynStatus status = asynSuccess, tmp;
	epicsInt32 addr, xch, xiastatus, i, dummy, realnbins, modulus;
	xmap_channel_t *chan;
	epicsFloat64 dblNbins;

	pasynManager->getAddr(pasynUser, &addr);
	chan = &(pPvt->chan[addr]);
	xch = pPvt->xch[addr];

	/* MCA number of bins on the xmap has a resolution of XMAP_MCA_BIN_RES (256)
	 * bins per channel. Since the user can enter any value we must cap this to
	 * the closest multiple of 256 */
	modulus = nbins % XMAP_MCA_BIN_RES;
	if (modulus == 0) realnbins = nbins;
	else if (modulus < XMAP_MCA_BIN_RES/2) realnbins = nbins - modulus;
	else realnbins = nbins + (XMAP_MCA_BIN_RES - modulus);

    nbins = realnbins;
	dblNbins = (epicsFloat64)nbins;

	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_nbins ch=%d module: %d nbins=%d real nbins=%d\n",
	        xch, pPvt->moduleNo ,nbins,dblNbins);

	if (nbins < XMAP_MIN_MCA_BINS || nbins > XMAP_MAX_MCA_BINS)
	{
		asynPrint(pasynUser, ASYN_TRACE_ERROR, "drvMcaXmap::xmap_mca_nbins nbins %d is out of range. Capping to min/max [%d...%d]\n",
				nbins, XMAP_MIN_MCA_BINS, XMAP_MAX_MCA_BINS);
		if (nbins > XMAP_MAX_MCA_BINS) nbins = XMAP_MAX_MCA_BINS;
		else if (nbins < XMAP_MIN_MCA_BINS) nbins = XMAP_MIN_MCA_BINS;
	}

	/* First check if we are acquiring on any of the channels
	 * that we are configuring (we cannot change nbins during a run)
	 */
	for(i = 0; i < XMAP_NCHANS_MODULE; i++)
	{
		if (pPvt->chan[i].acquiring == TRUE)
		{
			asynPrint(pasynUser, ASYN_TRACE_ERROR, "drvMcaXmap::xmap_mca_nbins ERROR: acquisition running on xch %d. Cannot set nbins during run\n",
			        pPvt->xch[i]);
			return asynError;
		}
	}

	/* Set the number of bins to all channels in the module.
	 * If we come across an error we latch it but attempt to continue
	 * to set the rest of the channels nbins. It is important that all
	 * channels nbins are in sync on the module as it affects the readout
	 * of the spectrums! */
	for(i = 0; i < XMAP_NCHANS_MODULE; i++)
	{
		xiastatus = xiaSetAcquisitionValues(pPvt->xch[i], "number_mca_channels", (void*)&dblNbins);
		tmp = xmap_xia_checkError(pasynUser, xiastatus, "number_mca_channels");
		if (tmp == asynError) status = tmp;
		else pPvt->chan[i].nBins = nbins;
	}
	/* Finally apply the operation to the board in order for it to take effect */
	xiastatus = xiaBoardOperation(pPvt->xch[0], "apply", (void*)&dummy);
	status = xmap_xia_checkError(pasynUser, xiastatus, "apply");

	return status;
}

static asynStatus xmap_mca_collect_module_data(xmap_driver_t* pPvt, asynUser* pasynUser)
{
	asynStatus status = asynSuccess;
	epicsInt32 addr, xiastatus, i, lastChNbins = 0;
	xmap_channel_t *chan;

	pasynManager->getAddr(pasynUser, &addr);
	chan = &(pPvt->chan[addr]);

	/* check that all channels nBins are equal. If they are not we are screwed
	 * because collecting the module MCA data depends on it...
	 *  - The correct and good thing to do if nBins are not equal across
	 *  all channels on the module, would be to ask xmap what is the
	 *  number of bins - unfortunately this run-tim information is always
	 *  reported as 0 by the Handel library! (useless) */
	for(i = 0; i < XMAP_NCHANS_MODULE; i++)
	{
		chan = &(pPvt->chan[i]);
		if (i != 0 && chan->nBins != lastChNbins)
		{
			asynPrint(pasynUser, ASYN_TRACE_ERROR,
					"drvMcaXmap::xmap_mca_collect_module_data ERROR not all channels have defined"\
					" equal number of MCA bins\n");
			return asynError;
		}
		lastChNbins = chan->nBins;
	}

	/* Collect module_data from the current module - into pointer chan->mcaModuleData. */
/*	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_collect_module_data Collecting data into ptr: %p\n",(pPvt->chan[0].mcaModuleData));
*/	xiastatus = xiaGetRunData(pPvt->xch[0], "module_mca", (void*)(pPvt->chan[0].mcaModuleData));
	status = xmap_xia_checkError(pasynUser, xiastatus, "module_mca");
	if (status == asynError) return status;

/*	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_collect_module_data %d GOT MCA DATA!!!\n", pPvt->xch[0]);
*/
	/* Set the channel pointer to point to the right place in the array where
	 * the specific channel data should be available */
	for (i = 0; i < XMAP_NCHANS_MODULE; i++)
	{
		chan = &(pPvt->chan[i]);
		chan->mcaChData = chan->mcaModuleData + (i * chan->nBins);
/*    	asynPrint(pasynUser, ASYN_TRACE_FLOW, "     xch: %d ptr: 0x%p nbins: %d i: %d\n",
                pPvt->xch[i], chan->mcaChData, chan->nBins, i);
*/	}

	return status;

}

/** Change the run until preset value and type
 * The preset or run-until value and type can be changed and written to the hardware.
 * \param pPvt The internal driver structure for the board.
 * \param pasynUser Pointer to the asynUser, requesting the operation.
 * \param type Enum value to indicate which type of preset we want to change to. See drvMca.h for valid
 * 				values in the xmap_preset_enum.
 * 	\param presetvalue The number of seconds or counts (make sure counts are integer values) to set the
 * 						preset to.
 * 	\return asynStatus
 */
static asynStatus xmap_mca_change_preset(xmap_driver_t* pPvt, asynUser* pasynUser, epicsUInt32 type, epicsFloat64 presetvalue)
{
	asynStatus status = asynSuccess;
	epicsFloat64 presettype;
	epicsInt32 xiastatus;
	xmap_preset_enum preset_e = (xmap_preset_enum) type;

	asynPrint(pasynUser, ASYN_TRACE_FLOW, "drvMcaXmap::xmap_mca_change_preset Type=%d Val=%.3f\n", type, presetvalue);

	switch( preset_e )
	{
		case XMAP_PRESET_NONE:
			presettype = XMAP_PRESET_NONE;
			presetvalue = 0.0;
			break;
		case XMAP_PRESET_REAL:
			presettype = XMAP_PRESET_REAL;
			break;
		case XMAP_PRESET_LIVE:
			presettype = XMAP_PRESET_LIVE;
			break;
		case XMAP_PRESET_EVENTS:
			presettype = XMAP_PRESET_EVENTS;
			break;
		case XMAP_PRESET_TRIGGERS:
			presettype = XMAP_PRESET_TRIGGERS;
			break;
		default:
			status = asynError;
			asynPrint(pasynUser, ASYN_TRACE_ERROR,
						"drvMcaXmap::xmap_mca_change_preset ERROR Invalid prefix type %d\n", type);
			break;
	}
	/* set the type of preset (live time) */
	xiastatus = xiaSetAcquisitionValues(XMAP_ALL, "preset_type", (void*)&presettype);
	status = xmap_xia_checkError(pasynUser, xiastatus, "preset_type");
	if (status == asynError) return status;
	pPvt->preset_type = preset_e;

	/* set the value of the run (until) preset: double in seconds
	 * or counts in case of EVENTS or TRIGGERS */
	xiastatus = xiaSetAcquisitionValues(XMAP_ALL, "preset_value", (void*)&presetvalue);
	status = xmap_xia_checkError(pasynUser, xiastatus, "preset_value");
	if (status == asynError)
	{
		pPvt->preset_value = 0.0;
		return status;
	}
	pPvt->preset_value = presetvalue;

	return status;
}
